/* **********************************************************************************
 *
 * Copyright (c) Ascend.NET Project. All rights reserved.
 *
 * This source code is subject to terms and conditions of the Shared Source License
 * for Ascend. A copy of the license can be found in the License.html file
 * at the root of this distribution. If you can not locate the Shared Source License
 * for Ascend, please send an email to ascendlic@<TBD>.
 * By using this source code in any fashion, you are agreeing to be bound by
 * the terms of the Shared Source License for Ascend.NET.
 *
 * You must not remove this notice, or any other, from this software.
 *
 * **********************************************************************************/

using Ascend.Resources;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Ascend.Windows.Forms.Design
{
    /// <summary>
    /// Provides the base class for types that define a list of items used to create a DesignerActionPanel.
    /// </summary>
    public class GradientCaptionActionList : DesignerActionList
    {
        private GradientCaption _gradientCaption;

        /// <summary>
        /// Initializes a new instance of the GradientCaptionActionList class.
        /// </summary>
        /// <param name="gradientCaption">A GradientCaption component.</param>
        public GradientCaptionActionList(GradientCaption gradientCaption) : base(gradientCaption)
        {
            this._gradientCaption = gradientCaption;

        }

        /// <summary>
        /// Specifies if the control is anti-aliased when drawn.
        /// </summary>
        public bool AntiAlias
        {
            get
            {
                return this._gradientCaption.AntiAlias;

            }

            set
            {
                this.GetPropertyByName("AntiAlias").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// Specifies the position and manner in which a control is docked.
        /// </summary>
        /// <remarks>
        /// <para>
        /// When a control is docked to an edge of its container, it is always positioned flush against that edge when the container is resized. If more than one control is docked to an edge, the controls appear side by side according to their z-order; controls higher in the z-order are positioned farther from the container's edge.
        /// </para>
        /// <para>
        /// If Left, Right, Top, or Bottom is selected, the specified and opposite edges of the control are resized to the size of the containing control's corresponding edges. If Fill is selected, all four sides of the control are resized to match the containing control's edges.
        /// </para>
        /// </remarks>
        /// <value>
        /// <para>
        /// System.Windows.Forms.DockStyle
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        public DockStyle Dock
        {
            get
            {
                return (DockStyle)this._gradientCaption.Dock;

            }

            set
            {
                this.GetPropertyByName("Dock").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The font associated with the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Font .
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        public Font Font
        {
            get
            {
                return this._gradientCaption.Font;

            }

            set
            {
                this.GetPropertyByName("Font").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// Gets or sets the gradient high (lighter) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient high color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        public Color GradientHighColor
        {
            get
            {
                return this._gradientCaption.GradientHighColor;

            }

            set
            {
                this.GetPropertyByName("GradientHighColor").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// Gets or sets the gradient low (darker) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient low color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        public Color GradientLowColor
        {
            get
            {
                return this._gradientCaption.GradientLowColor;

            }

            set
            {
                this.GetPropertyByName("GradientLowColor").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The text associated with the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.String .
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        public string Text
        {
            get
            {
                return (string)this._gradientCaption.Text;

            }

            set
            {
                this.GetPropertyByName("Text").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The ContentAlignment associated with this controls text.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.ContentAlignment . The ContentAlignment associated with this controls text.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        public ContentAlignment TextAlign
        {
            get
            {
                return this._gradientCaption.TextAlign;

            }

            set
            {
                this.GetPropertyByName("TextAlign").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The image associated with the control.
        /// </summary>
        /// <value>
        /// Image
        /// </value>
        public Image Image
        {
            get
            {
                return (Image)this._gradientCaption.Image;

            }

            set
            {
                if (value != null && value.ToString() == "(none)")
                {
                    return;

                }

                this.GetPropertyByName("Image").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The ContentAlignment associated with this controls image.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.ContentAlignment . The ContentAlignment associated with this controls image.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        public ContentAlignment ImageAlign
        {
            get
            {
                return this._gradientCaption.ImageAlign;

            }

            set
            {
                this.GetPropertyByName("ImageAlign").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// The name of the control
        /// </summary>
        /// <value>
        /// System.String
        /// </value>
        public string Name
        {
            get
            {
                return (string)this._gradientCaption.Name;

            }

            set
            {
                this.GetPropertyByName("Name").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// Gets or sets the painting style applied to the background in a control.
        /// </summary>
        /// <value>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        public RenderMode RenderMode
        {
            get
            {
                return this._gradientCaption.RenderMode;

            }

            set
            {
                this.GetPropertyByName("RenderMode").SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns></returns>
        private PropertyDescriptor GetPropertyByName(string propertyName)
        {
            PropertyDescriptor propertyDescriptor;

            propertyDescriptor = TypeDescriptor.GetProperties(this._gradientCaption)[propertyName];

            if (propertyDescriptor == null)
            {
                throw new ArgumentException(ResourceText.GetLocalizedString("GCPropertyNotFoundExceptionText"));

            }
            else
            {
                return propertyDescriptor;

            }

        }

        private void ChangeProperty(string propertyName, object value)
        {
            PropertyDescriptor propertyDescriptor = TypeDescriptor.GetProperties(this._gradientCaption)[propertyName];
            if (propertyDescriptor != null)
            {
                propertyDescriptor.SetValue(this._gradientCaption, value);

            }

        }

        /// <summary>
        /// Returns the collection of DesignerActionItem objects contained in the list. 
        /// </summary>
        /// <returns>A DesignerActionItem array that contains the items in this list.</returns>
        public override DesignerActionItemCollection GetSortedActionItems()
        {
            DesignerActionItemCollection items = new DesignerActionItemCollection();

            items.Add(new DesignerActionPropertyItem("Name", ResourceText.GetLocalizedString("ActionListDisplayNameName"), "Design", ResourceText.GetLocalizedString("NameDescription")));

            items.Add(new DesignerActionHeaderItem(ResourceText.GetLocalizedString("ActionListAppearanceCategory"), "Appearance"));
            items.Add(new DesignerActionTextItem(ResourceText.GetLocalizedString("ActionListAppearanceTextItem"), "Appearance"));
            items.Add(new DesignerActionPropertyItem("AntiAlias", ResourceText.GetLocalizedString("ActionListDisplayNameAntiAlias"), "Appearance", ResourceText.GetLocalizedString("AntiAliasDescription")));
            items.Add(new DesignerActionPropertyItem("RenderMode", ResourceText.GetLocalizedString("ActionListDisplayNameRenderMode"), "Appearance", ResourceText.GetLocalizedString("RenderModeDescription")));
            items.Add(new DesignerActionPropertyItem("GradientHighColor", ResourceText.GetLocalizedString("ActionListDisplayNameGradientHighColor"), "Appearance", ResourceText.GetLocalizedString("GradientHighColorDescription")));
            items.Add(new DesignerActionPropertyItem("GradientLowColor", ResourceText.GetLocalizedString("ActionListDisplayNameGradientLowColor"), "Appearance", ResourceText.GetLocalizedString("GradientLowColorDescription")));
            items.Add(new DesignerActionPropertyItem("Font", ResourceText.GetLocalizedString("ActionListDisplayNameFont"), "Appearance", ResourceText.GetLocalizedString("FontDescription")));
            items.Add(new DesignerActionPropertyItem("Image", ResourceText.GetLocalizedString("ActionListDisplayNameImage"), "Appearance", ResourceText.GetLocalizedString("ImageDescription")));
            items.Add(new DesignerActionPropertyItem("ImageAlign", ResourceText.GetLocalizedString("ActionListDisplayNameImageAlign"), "Appearance", ResourceText.GetLocalizedString("ImageAlignmentDescription")));
            items.Add(new DesignerActionPropertyItem("Text", ResourceText.GetLocalizedString("ActionListDisplayNameText"), "Appearance", ResourceText.GetLocalizedString("TextDescription")));
            items.Add(new DesignerActionPropertyItem("TextAlign", ResourceText.GetLocalizedString("ActionListDisplayNameTextAlign"), "Appearance", ResourceText.GetLocalizedString("TextAlignmentDescription")));

            items.Add(new DesignerActionHeaderItem(ResourceText.GetLocalizedString("ActionListLayoutCategory"), "Layout"));
            items.Add(new DesignerActionTextItem(ResourceText.GetLocalizedString("ActionListLayoutTextItem"), "Layout"));
            items.Add(new DesignerActionPropertyItem("Dock", ResourceText.GetLocalizedString("ActionListDisplayNameDock"), "Layout", ResourceText.GetLocalizedString("DockedDescription")));

            return items;

        }

    }

}
