/* **********************************************************************************
 *
 * Copyright (c) TMF.AscendedThemeKit.Unthemed.NET Project. All rights reserved.
 *
 * This source code is subject to terms and conditions of the Shared Source License
 * for TMF.AscendedThemeKit.Unthemed. A copy of the license can be found in the License.html file
 * at the root of this distribution. If you can not locate the Shared Source License
 * for TMF.AscendedThemeKit.Unthemed, please send an email to ascendlic@<TBD>.
 * By using this source code in any fashion, you are agreeing to be bound by
 * the terms of the Shared Source License for TMF.AscendedThemeKit.Unthemed.NET.
 *
 * You must not remove this notice, or any other, from this software.
 *
 * **********************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Text;

namespace TMF.AscendedThemeKit.Unthemed
{
    /// <summary>
    /// Represents border information associated with a user interface (UI) element.
    /// </summary>
    [StructLayout(LayoutKind.Sequential), TypeConverter(typeof(BorderConverter))]
    public struct Border
    {
        /// <summary>
        /// Provides a Border object with no borders.
        /// </summary>
        public static readonly Border Empty;

        private bool _all;
        private int _bottom;
        private int _left;
        private int _right;
        private int _top;

        /// <summary>
        /// Initializes a new instance of the Border class using the supplied border size for all edges.
        /// </summary>
        /// <param name="all">System.Int32. The number of pixels to be used the border for all edges.</param>
        /// <remarks>
        /// This constructor sets the Right, Left, Bottom, Top and All properties to the value of the all parameter.
        /// </remarks>
        public Border(int all)
        {
            this._all = true;

            this._left = all;
            this._top = all;
            this._right = all;
            this._bottom = all;

        }

        /// <summary>
        /// Initializes a new instance of the Border class using the supplied border sizes for the edges.
        /// </summary>
        /// <param name="left">System.Int32. The border size, in pixels, for the left edge.</param>
        /// <param name="top">System.Int32. The border size, in pixels, for the top edge.</param>
        /// <param name="right">System.Int32. The border size, in pixels, for the right edge.</param>
        /// <param name="bottom">System.Int32. The border size, in pixels, for the bottom edge.</param>
        /// <remarks>
        /// This constructor sets the Right, Left, Bottom, Top properties. If the parameters are all the same the All parameter is also set to the value.
        /// </remarks>
        public Border(int left, int top, int right, int bottom)
        {
            this._left = left;
            this._top = top;
            this._right = right;
            this._bottom = bottom;

            this._all = (!(this._top != this._left) && !(this._top != this._right)) && (this._top == this._bottom);

        }

        /// <summary>
        /// Gets or sets the border value for all the edges.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Int32 . The border, in pixels, for all edges if the same; otherwise, -1.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [RefreshProperties(RefreshProperties.All)]
        public int All
        {
            get
            {
                if (!this._all)
                {
                    return -1;

                }

                return this._top;

            }

            set
            {
                if (this._all && (this._top == value))
                {
                    return;

                }

                this._all = true;

                this._bottom = value;
                this._right = value;
                this._left = value;
                this._top = value;

            }

        }

        /// <summary>
        /// Gets or sets the border value for the bottom edge.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Int32 . The border, in pixels, for the bottom edge.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        /// <remarks>
        /// Setting this value can also alter the All property.
        /// </remarks>
        [RefreshProperties(RefreshProperties.All)]
        public int Bottom
        {
            get
            {
                return this._bottom;

            }

            set
            {
                this._bottom = value;

            }

        }

        /// <summary>
        /// Gets or sets the border value for the left edge.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Int32 . The border, in pixels, for the left edge.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        /// <remarks>
        /// Setting this value can also alter the All property.
        /// </remarks>
        [RefreshProperties(RefreshProperties.All)]
        public int Left
        {
            get
            {
                return this._left;

            }

            set
            {
                this._left = value;

            }

        }

        /// <summary>
        /// Gets or sets the border value for the right edge.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Int32 . The border, in pixels, for the right edge.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        /// <remarks>
        /// Setting this value can also alter the All property.
        /// </remarks>
        [RefreshProperties(RefreshProperties.All)]
        public int Right
        {
            get
            {
                return this._right;

            }

            set
            {
                this._right = value;

            }

        }

        /// <summary>
        /// Gets or sets the border value for the top edge.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Int32 . The border, in pixels, for the top edge.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        /// <remarks>
        /// Setting this value can also alter the All property.
        /// </remarks>
        [RefreshProperties(RefreshProperties.All)]
        public int Top
        {
            get
            {
                return this._top;

            }

            set
            {
                this._top = value;

            }

        }

        /// <summary>
        /// Returns if the All property has been used.
        /// </summary>
        /// <returns>True or false</returns>
        public bool ShouldSerializeAll()
        {
            return this._all;

        }

        /// <summary>
        /// Determines whether the specified object is equal to the current Object.
        /// </summary>
        /// <param name="obj">The Object to compare with the current Object.</param>
        /// <returns>true if the specified Object is equal to the current Object; otherwise, false.</returns>
        public override bool Equals(object obj)
        {
            if (obj is Border)
            {
                Border border = (Border)obj;
                if (((border._all == this._all) && (border._top == this._top)) && ((border._left == this._left) && (border._bottom == this._bottom)))
                {
                    return (border._right == this._right);

                }

            }

            return false;

        }

        /// <summary>
        /// Serves as a hash function for the border type, suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        /// <returns>
        /// <para>
        /// System.Int32
        /// </para>
        /// <para>
        /// A hash code for the Border object.
        /// </para>
        /// </returns>
        public override int GetHashCode()
        {
            return (((this.Left ^ RotateLeft(this.Top, 8)) ^ RotateLeft(this.Right, 0x10)) ^ RotateLeft(this.Bottom, 0x18));

        }

        /// <summary>
        /// Returns a string that contains the border values for an instance of the Border class.
        /// </summary>
        /// <returns>
        /// <para>
        /// System.String . 
        /// </para>
        /// <para>
        /// A String that represents the current Border.
        /// </para>
        /// </returns>
        /// <remarks>
        /// <para>
        /// This method returns a string containing the labeled values of the border for all four edges.
        /// </para>
        /// <para>
        /// This method overrides ToString.
        /// </para>
        /// </remarks>
        public override string ToString()
        {
            string[] textArray = new string[9] { "{Left=", this.Left.ToString(CultureInfo.CurrentCulture.NumberFormat), ",Top=", this.Top.ToString(CultureInfo.CurrentCulture.NumberFormat), ",Right=", this.Right.ToString(CultureInfo.CurrentCulture.NumberFormat), ",Bottom=", this.Bottom.ToString(CultureInfo.CurrentCulture.NumberFormat), "}" };
            return string.Concat(textArray);

        }

        /// <summary>
        /// Performs vector addition on the two specified Border objects, resulting in a new Border.
        /// </summary>
        /// <param name="border1">Border. The first Border to add.</param>
        /// <param name="border2">Border. The second Border to add.</param>
        /// <returns>Border. A new Border that results from adding border1 and border2.</returns>
        public static Border Add(Border border1, Border border2)
        {
            return (border1 + border2);

        }

        /// <summary>
        /// Performs vector subtraction on the two specified Border objects, resulting in a new Border.
        /// </summary>
        /// <param name="border1">Border. The first Border to subtract.</param>
        /// <param name="border2">Border. The second Border to subtract.</param>
        /// <returns>Border. A new Border that results from subtracting border2 from border1.</returns>
        public static Border Subtract(Border border1, Border border2)
        {
            return (border1 - border2);

        }

        private static int RotateLeft(int value, int bits)
        {
            bits = bits % 0x20;
            return ((value << (bits & 0x1f)) | (value >> ((0x20 - bits) & 0x1f)));

        }

        /// <summary>
        /// Performs vector addition on the two specified Border objects, resulting in a new Border.
        /// </summary>
        /// <param name="border1">Border. The first Border to add.</param>
        /// <param name="border2">Border. The second Border to add.</param>
        /// <returns>Border. A new Border that results from adding border1 and border2.</returns>
        public static Border operator +(Border border1, Border border2)
        {
            return new Border((border1.Left + border2.Left), (border1.Top + border2.Top), (border1.Right + border2.Right), (border1.Bottom + border2.Bottom));

        }

        /// <summary>
        /// Performs vector subtraction on the two specified Border objects, resulting in a new Border.
        /// </summary>
        /// <param name="border1">Border. The first Border to subtract.</param>
        /// <param name="border2">Border. The second Border to subtract.</param>
        /// <returns>Border. A new Border that results from subtracting border2 from border1.</returns>
        public static Border operator -(Border border1, Border border2)
        {
            return new Border(border1.Left - border2.Left, border1.Top - border2.Top, border1.Right - border2.Right, border1.Bottom - border2.Bottom);

        }

        /// <summary>
        /// Tests whether two specified Border objects are not equivalent.
        /// </summary>
        /// <param name="border1">Border. A Border to test.</param>
        /// <param name="border2">Border. A Border to test.</param>
        /// <returns>System.Boolean . true if the two Border objects are different; otherwise, false.</returns>
        public static bool operator !=(Border border1, Border border2)
        {
            return !(border1.Equals(border2));

        }

        /// <summary>
        /// Tests whether two specified Border objects are equivalent.
        /// </summary>
        /// <param name="border1">Border. A Border to test.</param>
        /// <param name="border2">Border. A Border to test.</param>
        /// <returns>System.Boolean . true if the two Border objects are equivalent; otherwise, false.</returns>
        public static bool operator ==(Border border1, Border border2)
        {
            return border1.Equals(border2);

        }

    }

}
